@@ -63,6 +63,10 @@ CREATE [% keyword %] [% name %][% child_output %]
 );
 [% END %]
 
+[% BLOCK pk_text %]
+    PRIMARY KEY( [% FOREACH pk IN pks %][% pk %][% UNLESS loop.last %], [% END %][% END %] )
+[%- END -%]
+
 [% BLOCK table_element_block %]    [% name %] [% child_output %][% END %]
 
 [% BLOCK field_statement %]
@@ -107,6 +111,7 @@ use strict; use warnings;
 sub output_sql_mysql {
     my $self         = shift;
     my $child_output = shift;
+    my $lookup       = shift;
 
     return if ( $self->_skip_this_block );
 
@@ -116,6 +121,15 @@ sub output_sql_mysql {
         push @{ $output{ $type } }, $output;
     }
 
+    my $pks = $self->find_primary_key( $self->{__NAME__}, $lookup );
+
+    if ( ref( $pks ) eq 'ARRAY' ) { # multi-column primary key
+        my $pk_text = Bigtop::Backend::SQL::MySQL::pk_text(
+                { pks => $pks, }
+        );
+        push @{ $output{ table_body } }, $pk_text;
+    }
+
     my $child_out_str = Bigtop::Backend::SQL::MySQL::table_body(
         { child_output => $output{table_body} }
     );
@@ -184,12 +198,22 @@ use strict; use warnings;
 
 my %expansion_for = (
     int4               => 'MEDIUMINT',
-    primary_key        => 'PRIMARY KEY',
     assign_by_sequence => 'AUTO_INCREMENT',
     auto               => 'AUTO_INCREMENT',
     varchar            => 'VARCHAR(100)',
 );
 
+sub mysql_pk_text {
+    my $self   = shift;
+    my $lookup = shift;
+    my $table  = $self->get_table_name();
+
+    my $pks    = table_block->find_primary_key( $table, $lookup );
+
+    return ( ref( $pks ) eq 'ARRAY' ) ? '' : 'PRIMARY KEY';
+}
+
+
 sub output_sql_mysql {
     my $self   = shift;
     shift;  # there is no child output
@@ -201,7 +225,11 @@ sub output_sql_mysql {
     foreach my $arg ( @{ $self->{__DEF__}{__ARGS__} } ) {
         my $expanded_form = $expansion_for{$arg};
 
-        if ( defined $expanded_form ) {
+        if ( $arg eq 'primary_key' ) {
+            my $pk_text = $self->mysql_pk_text( $lookup );
+            push @keywords, $pk_text if $pk_text;
+        }
+        elsif ( defined $expanded_form ) {
             push @keywords, $expanded_form;
         }
         else {
